    member

    include('trackbar.inc')

HandlerQueue    queue
qHwnd             HWND
qControlHwnd      HWND
qControl          &cControl
                end

mWndProc        long(0), thread

    map
      module('Windows')
        CreateWindowEx(DWORD,*LPCSTR,*LPCSTR, DWORD, SIGNED, SIGNED, SIGNED, SIGNED, HWND, HMENU, HINSTANCE, LPVOID), HWND, PASCAL, RAW, name('CreateWindowExA')
        SendMessage(HWND,UNSIGNED,WPARAM,LPARAM),LRESULT,PASCAL,NAME('SendMessageA')
        CallWindowProc(LONG,HWND,unsigned,WPARAM,LPARAM),LONG,RAW,PASCAL,NAME('CallWindowProcA')
        SetWindowLong(HWND,signed,LONG),LONG,RAW,PASCAL,NAME('SetWindowLongA')
        InitCommonControls(), PASCAL, NAME('InitCommonControls')
        DefWindowProc(HWND,UNSIGNED,WPARAM,LPARAM),LRESULT,PASCAL,NAME('DefWindowProcA')
      end
      TBTestWinProc(unsigned MyHwnd, unsigned usMsg, UNSIGNED WParam, LONG lParam),LONG,PASCAL
    end


cControl.WndProc    function(unsigned MyhWnd, unsigned usMsg, unsigned WParam, long lParam)

    code
    return(true)

cControl.CheckHwnd  function()

    code
    if ~SELF.m_Hwnd
      return(false)
    end
    return(true)

cControl.Notification   function()

    code
    return(SELF.m_Notification)


! --------------------------------------------------------------
cTrackBar.CreateTB      function(long pRegion, unsigned pParentHwnd)
! --------------------------------------------------------------
! This method creates the trackbar control and return true if the control was
! created successfully.
! pRegion is a FEQ of a region control to get a screen location on the window
! pParent is the Parent Handle of the control. The control will send notification
! message to that hanlde.

    code
    ! First, make sure to make Windows happy and call the common controls init code.
    InitCommonControls()

    ! Is there a style defined? if not, give a default.
    if ~SELF.m_Style
      SELF.m_Style = bor(WS_CHILD, bor(WS_TABSTOP, bor(WS_VISIBLE, TBS_AUTOTICKS)))
    end

    ! Check if the parent window has prop:pixels set to true
    if ~0{prop:pixels}
      0{prop:pixels} = true
    end

    ! Create the control
    SELF.m_Hwnd = CreateWindowEx(SELF.m_StyleEx, |
                               SELF.m_ClassName, |
                               SELF.m_cWindowName, |
                               SELF.m_Style , |
                               pRegion{prop:XPos} + 2, |
                               pRegion{prop:YPos} + 2, |
                               pRegion{prop:Width} - 2, |
                               pRegion{prop:Height} - 2, |
                               pParentHwnd, |
                               SELF.m_ControlID, 0, 0)


    ! Check if the HWND is valid.
    if ~SELF.CheckHwnd()
      return(false)
    end

    ! Check if the window proc has already been subclassed
    if ~mWndProc
      mWndProc = SetWindowLong(pParentHwnd, GWL_WndProc, ADDRESS(TBTestWinProc))
    end

    ! Grab the current thread number and address of the window proc
    SELF.m_ThreadID = thread()
    SELF.m_pWndProc = mWndProc

    ! Now, populate the handler queue with appropriate value.
    HandlerQueue.qHwnd = pParentHwnd
    HandlerQueue.qControl &= SELF
    HandlerQueue.qControlHwnd = SELF.m_hwnd

    add(HandlerQueue, HandlerQueue.qHwnd, HandlerQueue.qControlHwnd)
    if errorcode()
      return(false)
    end

    ! We are doing good so far! Now kill the region control
    destroy(pRegion)
    return(true)

! ------------------------------------
cTrackBar.GetLowerLimit     function()
! ------------------------------------
! Return the Lower limit of the control. This is
! the lower value the Slider can be positioned over.

    code
    return(SELF.m_Lower)

! ------------------------------------
cTrackBar.GetUpperLimit     function()
! ------------------------------------
! Return the UPPER limit of the control. This is
! the upper value the Slider can be positioned over.

    code
    return(SELF.m_Upper)


! --------------------------------
cTrackBar.GetStart      function()
! --------------------------------
! Return the Start value of the range
! defined in the Channel Rectangle of the control.

    code
    if ~SELF.CheckHWnd()
      return(0)
    end
    return(SendMessage(SELF.m_Hwnd, TBM_GETSELSTART, 0, 0))

! ------------------------------
cTrackBar.GetEnd      function()
! ------------------------------
! Return the End value of the range
! defined in the Channel Rectangle of the control.

    code
    if ~SELF.CheckHWnd()
      return(0)
    end
    return(SendMessage(SELF.m_Hwnd, TBM_GETSELEND, 0, 0))

! ------------------------------
cTrackBar.GetPos      function()
! ------------------------------

    code
    return(SendMessage(SELF.m_Hwnd, TBM_GETPOS, 0, 0))

! ---------------------------------------
cTrackBar.SetPos      function(long pPos)
! ---------------------------------------

    code
    if SendMessage(SELF.m_Hwnd, TBM_SETPOS, true, pPos).  ! The true tells the control to repaint itself.
    return(true)

! ----------------------------------------------------
cTrackBar.SetLimit  function(long pLower, long pUpper)
! ----------------------------------------------------
! Set the limit of the trackbar control. The limit is the
! range of values the control can represent.

gLParam     group
lWord         short
hWord         short
            end

lParam      long, over(gLParam)

    code
    assert(SELF.m_hwnd)
    if ~SELF.CheckHWnd()
      return(false)
    end

    gLParam.lWord = pLower
    gLParam.hWord = pUpper
    SELF.m_Lower = pLower
    SELF.m_Upper = pUpper
  
    if SendMessage(SELF.m_Hwnd, TBM_SETRANGE, true, lParam). ! The true will tell the control to repaint itself.

    return(true)


! ----------------------------------------------
cTrackBar.SetPageSize       function(long pPage)
! ----------------------------------------------
! SetPageSize will set the number of position the Slider will
! move when the user click on the slider bar

    code
    assert(SELF.m_HWnd)
    if ~SELF.CheckHWnd()
      return(false)
    end

    if SendMessage(SELF.m_Hwnd, TBM_SETPAGESIZE, 0, pPage).
    return(true)


! ------------------------------------------------------
cTrackBar.SetRange      function(long pStart, long pEnd)
! ------------------------------------------------------
! SetRange will define the range displayed in the Channel Rectangle
! of the control. The control must have the TBS_ENABLESELRANGE bit style set
! to properly use this method.

gLParam     group
lWord         short
hWord         short
            end

lParam      long, over(gLParam)

    code
    assert(SELF.m_Hwnd)
    if ~SELF.CheckHwnd()
      return(false)
    end

    gLParam.lWord = pStart
    gLParam.hWord = pEnd
    if SendMessage(SELF.m_Hwnd, TBM_SETSEL, true, lParam). ! The true will tell the control to repaint itself.
    return(true)

! ------------------------------------------
cTrackBar.SetTicsFreq   function(long pFreq)
! ------------------------------------------
! SetTicsFreq will tell the control to place tics marks
! every pFreq along the control

    code
    assert(SELF.m_Hwnd)
    if ~SELF.CheckHwnd()
      return(false)
    end
    if SendMessage(SELF.m_HWnd, TBM_SETTICFREQ, pFreq, 0).
    return(true)

! -----------------------------------------
cTrackBar.SetTicAt      function(long pTic)
! -----------------------------------------
! SetTicAt will place a tic mark at the position
! specified by the pTic parameter.

    code
    if SendMessage(SELF.m_Hwnd, TBM_SETTIC, 0, pTic) = -1
      return(false)
    end
    return(true)


cTrackBar.TakeNotification      function()

    code
    return(true)

! -----------------------------------------------------------------------------------------
cTrackBar.WndProc       function(unsigned MyhWnd, unsigned usMsg, unsigned WParam, long lParam)
! -----------------------------------------------------------------------------------------
! Window procedure. This is the procedure called by the subclassed window procedure of the parent.
! The idea is to check if there is a child HWND associated with the parent HWND. Those are collected in
! a queue. A simple lookup in this queue will then retrieve a reference to the object and a call
! to the Object.WndProc (this method) will be executed.

gWParam     group
lWord         short
hWord         short
            end

wPar        ulong, over(gWParam)

        code
        wPar = WParam
        case usMsg
        of WM_HSCROLL
        orof WM_VSCROLL
          SELF.m_Notification = gWParam.lWord
          ! By default, we want to get the slider position
          case gWParam.lWord
          of TB_THUMBPOSITION
          orof TB_THUMBTRACK
            SELF.m_SliderPos = gWParam.hWord
          else
            SELF.m_SliderPos = SendMessage(lParam, TBM_GETPOS, 0, 0)
          end
          ! And whatever we want to do.
          if ~SELF.TakeNotification()
            return(false)
          end
          post(0, , SELF.m_ThreadID) ! This will wake up the accept loop of the parent window
        end
        return(CallWindowProc(self.m_pWndProc, MyHwnd, usMsg, WParam, lParam))


TBTestWinProc        PROCEDURE  (unsigned MyHWnd, unsigned usMsg,UNSIGNED WParam,LONG lParam) ! Declare Procedure
  CODE
  ! Look in the queue if there is a corresponding object to handle
  ! this message
  HandlerQueue.qHWnd = MyHwnd
  HandlerQueue.qControlHwnd = lParam
  get(HandlerQueue, HandlerQueue.qhWnd, HandlerQueue.qControlHwnd)
  if ~errorcode()
    ! An object is there? Call its WndProc
    return(HandlerQueue.qControl.WndProc(MyHwnd, usMsg, WParam, lParam))
  end
  ! Else return the default window procedure
  return(CallWindowProc(mWndProc, MyHwnd, usMsg, WParam, lParam))